#!/usr/bin/env python
'''
Created on Jan. 26, 2015

@author: Tung Nguyen <nltung@gmail.com>
'''
import sys, os, time, multiprocessing, optparse
import subprocess, logging, datetime

def parse_config(config_file):
  singleAln, partitionAln, partOpts, genericOpts = [], [], [], []
  with open(config_file) as f:
    #lines = f.readlines()
    lines = [line.strip() for line in f if line.strip()]
  readSingleAln = False
  readPartAln = False
  partOpt = False
  genericOpt = False
  for line in lines:
    #print line
    if line == 'START_SINGLE_ALN':
      readSingleAln = True
      continue
    if line == 'END_SINGLE_ALN':
      readSingleAln = False
      continue
    if readSingleAln:
      singleAln.append(line)
    if line == 'START_PARTITION_ALN':
      readPartAln = True
      continue
    if line == 'END_PARTITION_ALN':
      readPartAln = False
      continue
    if readPartAln:
      partitionAln.append(line.split())
    if line == 'START_PARTITION_OPTIONS':
      partOpt = True
      continue
    if line == 'END_PARTITION_OPTIONS':
      partOpt = False
      continue
    if line == 'START_GENERIC_OPTIONS':
      genericOpt = True
      continue
    if line == 'END_GENERIC_OPTIONS':
      genericOpt = False
      continue
    if partOpt:
      partOpts.append(line)
    if genericOpt:
      genericOpts.append(line)
  return (singleAln, partitionAln, genericOpts, partOpts)


if __name__ == '__main__':
  usage = "USAGE: %prog [options]"
  parser = optparse.OptionParser(usage=usage)
  parser.add_option('-b','--binary', dest="iqtree_bin", help='Path to your IQ-TREE binary')
  parser.add_option('-c','--config', dest="config_file", help='Path to test configuration file')
  parser.add_option('-o', '--output', dest="outFile", help='Output file for test cases')
  parser.add_option('-f', '--flags', dest="flags", help='Additional flags for IQ-TREE')
  (options, args) = parser.parse_args()
  if not options.iqtree_bin or not options.config_file:
    parser.print_help()
    exit(0)
  (singleAln, partitionAln, genericOpts, partOpts) = parse_config(options.config_file)
  testCmds = []
  # Generate test commands for single model
  for aln in singleAln:
    for opt in genericOpts:
      cmd = '-s ' + aln + ' -redo ' + opt
      if options.flags:
        cmd = cmd + ' ' + options.flags
        testCmds.append(cmd)
  # Generate test commands for partition model
  for aln in partitionAln:
    for opt in genericOpts:
      for partOpt in partOpts:
        cmd = '-s ' + aln[0] + ' -redo ' + opt + ' ' + partOpt + ' ' + aln[1]
        if options.flags:
            cmd = cmd + ' ' + options.flags
        testCmds.append(cmd)

  testNr = 1
  jobs = []
  for cmd in testCmds:
    testIDRel = os.path.basename(options.iqtree_bin) + "_TEST_" + str(testNr)
    testCMD = testIDRel + " " + os.path.abspath(options.iqtree_bin) + " -pre " + testIDRel + " " + cmd
    testNr = testNr + 1
    jobs.append(testCMD)
#  print "\n".join(jobs)
  outfile = open(options.outFile, "wb")
  for job in jobs:
    print >> outfile, job
  outfile.close()



#!/usr/bin/env python
'''
Created on Feb. 01, 2015
This script collects all commands submited by users of the IQ-TREE web service 
that were crashed
Default location: /project/web-iqtree/user-data 

@author: Tung Nguyen
@email: nltung@gmai.com
'''
import sys, os, time, multiprocessing, optparse, fnmatch 
import subprocess, logging, datetime
import cmd
import shutil
from operator import itemgetter
from cmd import Cmd

def collect_logs(log_dir):
    bugLogs = []
    numLog = 0
    for root, dirnames, filenames in os.walk(log_dir):
        for filename in fnmatch.filter(filenames, '*.log'):
            numLog = numLog + 1
            logFile = os.path.join(root,filename)
            if 'CRASH' in open(logFile).read():
                bugLogs.append(logFile)
    return (bugLogs, numLog)

def collect_cmds(logFiles, out_dir):
    if not os.path.exists(out_dir):
        #shutil.rmtree(options.out_dir)
        os.makedirs(out_dir)
    runs = []
    for log in logFiles:
        id = log.split('/')[-2]
        email = log.split('/')[-3]
        with open(log) as f:            
            for line in f:
                if line.startswith('Command:'):
                    cmd = " ".join(line.split()[2:])
                    aln = line.split()[3]
                    run_dir = os.path.abspath(os.path.join(log, os.pardir))
                    shutil.copy2(os.path.join(run_dir, aln), out_dir)
                    if line.find('-sp') != -1:
                        partitionFile = line.split()[5]
                        shutil.copy2(os.path.join(run_dir, partitionFile), out_dir)
                if line.startswith('Seed:'):
                    seed = line.split()[1]
                    runs.append((int(id), email,seed, aln, cmd))
                    #print run_dir
                    break
    return runs

def create_test_cmds(runs, iqtree_binary, filename):
    outfile = open(filename, "wb")
    for run in runs:
        run_id = run[1] + "_" + str((run[0]))
        seed = run[2]
        args = run[4]
        if not " -b " in args:
            cmd = run_id + ' ' + iqtree_binary + ' ' + args + ' -seed ' + seed + ' -pre ' + run_id
            print >> outfile, cmd
    outfile.close()
                                        
if __name__ == '__main__':
    usage = "USAGE: %prog [options]"
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-d', '--indir', dest="in_dir", 
                      help='Path to user directory of the IQ-TREE web server [default: %default]', default="/project/web-iqtree/user-data/")
    parser.add_option('-o', '--outdir', dest="out_dir", 
                      help='Directory containing alignments [default: %default]', default="webserver_alignments")
    parser.add_option('-b', '--iqtree_bin', dest='iqtree_bin', help='Path to IQ-Tree binary')
    (options, args) = parser.parse_args()
    if not options.iqtree_bin:
        print "Please specify the path to your IQ-TREE binary"
        parser.print_help()
        exit(0)
    print "Collecting buggy runs from " + options.in_dir
    (bugLogs, numLog) = collect_logs(options.in_dir)
    print ("Found %d job submissions, %d of them caused bugs" % (numLog, len(bugLogs)))
    runs = collect_cmds(bugLogs, options.out_dir)
    runs.sort(key=lambda tup: tup[0])
    create_test_cmds(runs, options.iqtree_bin, os.path.basename(options.iqtree_bin) + '_test_webserver_cmds.txt')

    



#!/usr/bin/env python
'''
Created on Aug 23, 2014

@author: tung
'''
import sys, os, time, multiprocessing, optparse 
import subprocess, logging, datetime

def cpu_count():
    ''' Returns the number of CPUs in the system
    '''
    num = 1
    if sys.platform == 'win32':
        try:
            num = int(os.environ['NUMBER_OF_PROCESSORS'])
        except (ValueError, KeyError):
            pass
    elif sys.platform == 'darwin':
        try:
            num = int(os.popen('sysctl -n hw.ncpu').read())
        except ValueError:
            pass
    else:
        try:
            num = os.sysconf('SC_NPROCESSORS_ONLN')
        except (ValueError, OSError, AttributeError):
            pass

    return num

def exec_commands(cmds, name, num_cpus):
    ''' Exec commands in parallel in multiple process 
    (as much as we have CPU)
    '''
    if not cmds: return  # empty list

    def done(p):
        return p.poll() is not None
    def success(p):
        return p.returncode == 0
    def fail():
        sys.exit(1)
        
    # max_task = cpu_count()
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    my_time = datetime.datetime.now()
    handler = logging.FileHandler(name + "." + str(my_time.year) + str(my_time.month) + str(my_time.day) + 
                                  str(my_time.hour) + str(my_time.minute) + str(my_time.second) + ".log")
    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    max_task = multiprocessing.cpu_count()
    logger.info("Available CPUs = " + str(max_task) + " / using " + str(num_cpus) + " CPUs")
    logger.info("Number of jobs = " + str(len(cmds)))
    processes = []
    while True:
        while cmds and len(processes) < num_cpus:
            task = cmds.pop(0)
            #print subprocess.list2cmdline(task)
            task_id, cmd = task.split(" ", 1)
            logger.info("Executing job " + task_id + ": " + cmd.strip())
            #print cmd
            task_output = open(task_id + ".out", "w")
            time_cmd = "time " + cmd
            processes.append([subprocess.Popen(time_cmd, stderr=subprocess.STDOUT, stdout=task_output, shell=True), task_id])

        for p in processes:
            if done(p[0]):
                if success(p[0]):
                    #print "Process with ID = ", p.pid, " has finished"
                    #print "number of processes before removal: ", len(processes)
                    logger.info("Job " + p[1] + " has finished")
                    processes.remove(p)
                    #print "number of processes after removal: ", len(processes)
                else:
                    logger.info("Job " + p[1] + " finished with ERROR CODE " + str(p[0].returncode))
                    processes.remove(p)

        if not processes and not cmds:
            break
        else:
            time.sleep(5)
        
if __name__ == '__main__':
    max_cores = multiprocessing.cpu_count()
    usage = "USAGE: %prog [options]"
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-f','--cmd', dest="cmd", help='File containing all commands')
    parser.add_option('-c','--cpu', dest="cpu", help='Number of CPU to use', default=max_cores)
    (options, args) = parser.parse_args()
    if len(sys.argv) == 1:
        parser.print_help()
        exit(0)
    jobs = open(options.cmd, "r").readlines()
    exec_commands(jobs, options.cmd, int(options.cpu))
    
    


